/* $Id: Mv300Dev.c,v 1.4 91/02/06 11:39:56 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300Dev.c$
 *
 * $Revision: 1.4 $
 *
 * $State: Exp $
 *
 * $Author: cwcork $
 *
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log:	Mv300Dev.c,v $
 * Revision 1.4  91/02/06  11:39:56  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.3  91/01/26  09:04:30  cwcork
 * Replace $ with _ in all defined constants.
 * 
 * Revision 1.2  90/09/14  17:20:13  cwcork
 * Move hidden structure declaration for MV300DEV
 * into this module from Mv300Dev.h
 * 
 * Revision 1.1  90/08/01  14:55:50  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *   This is a generic control module for devices connected to
 *   the Motorola MVME300 IEEE-488 bus controller.
 *
 * FUNCTIONS:
 *  Exported-
 *   --Create and Forget--
 *   Mv300DevCreate	- Create and initialize the device control structure.
 *   Mv300DevForget	- Make device offline and free control structure memory.
 *   --Device IO functions--
 *   Mv300DevBrdIdGet   - Return board ID.
 *   Mv300DevBrdIdSet   - Set new board ID [Use with caution].
 *   Mv300DevClear	- Clear a device
 *   Mv300DevConfRd	- Read device configuration.
 *   Mv300DevConfWrt	- Write device configuration.
 *   Mv300DevLocal	- Enable local control for a device
 *   Mv300DevPPoll	- Parallel Poll a device
 *   Mv300DevPPollByte	- Configure a device for parallel poll
 *   Mv300DevRead	- Read device data.
 *   Mv300DevSPoll	- Return serial poll byte
 *   Mv300DevStatus	- Return board status
 *   Mv300DevTrigger    - Send GET to device.
 *   Mv300DevWrite	- Write device data.
 *  Local-
 *
 * COMMENTS:
 *
 * KNOWN BUGS:
 */
/*========================= External Declarations ============================*/
#define  MV300DEV_C

#include "vxWorks.h"
#include "memLib.h"
#include "Mv300Dev.h"

/* MV300_DEV: device control structure */
typedef struct 		/* MV300_DEV - MV300 controller structure */
{
  MV300_BRD_ID		brdId;		/* Board Identifier */
  SEM_ID		devSemId;	/* Device mutex semaphore */
  MV300_CONFIG		config;		/* Device configuration structure */
} MV300_DEV;

typedef MV300_DEV* MV300_DEV_ID;	/* The only public link to MV300_DEV */

/*========================= Local Declarations ===============================*/
/*========================= Local Macros =====================================*/
/*========================= Local Function Prototypes ========================*/
/*========================= Local Variables ==================================*/
/*========================= Exported Functions ===============================*/
/*------------------------- Create and Forget --------------------------------*/
/*-----------------------------------------------------------------------------
 * Mv300DevCreate (pDevId, brdId, pConfig) : status
 *
 * Purpose:
 *  Create and initialize the MVME300 device and control structure
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  pDevId	: MV300_DEV_ID*	- pointer to device control structure ID
 *  brdId	: MV300_BRD_ID  - board control structure ID.
 *  pConfig	: PMV300_CONFIG - pointer to device configuration structure.
 * Description:
 *  Create and initialize the MV300_DEV structure and initialize the associated
 *  device. 
 */
STATUS Mv300DevCreate (pDevId, brdId, pConfig)
     MV300_DEV_ID* pDevId;
     MV300_BRD_ID  brdId;
     PMV300_CONFIG pConfig;
{
  FAST MV300_DEV_ID devId;
  BYTE              commands[4];
  UINT		    bufLen;
  UINT		    bytesWritten;


  /* Try to create device control structure */
  if ((devId = (MV300_DEV_ID) malloc (sizeof (MV300_DEV))) == NULL)
    {
      *pDevId = devId;
      return (ERROR_EMEM);
    }

  /* Initialize module control structure */
  devId->brdId           = brdId;
  devId->devSemId	 = semCreate();
  devId->config.timeout	 = pConfig->timeout;
  devId->config.flags	 = pConfig->flags;
  devId->config.priAddr	 = pConfig->priAddr;
  devId->config.secAddr	 = pConfig->secAddr;
  devId->config.reosChar = pConfig->reosChar;
  devId->config.weosChar = pConfig->weosChar;

  /* Initialize device */
  if (pConfig->secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | pConfig->priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | pConfig->secAddr;
      commands[2] = MV300_BRD_K_GPIB_SDC;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | pConfig->priAddr;
      commands[1] = MV300_BRD_K_GPIB_SDC;
      commands[2] = MV300_BRD_K_GPIB_UNL;
      bufLen = 3;
    }
  Mv300BrdDevCommand (brdId, pConfig, commands, bufLen, &bytesWritten);

  /* Return structure id */
  *pDevId = devId;
  semGive (devId->devSemId);
  return (OK);
}/* Mv300DevCreate */

/*-----------------------------------------------------------------------------
 * Mv300DevForget (devId) : status
 *
 * Purpose:
 *  Offline and free memory
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  devId	: MV300_DEV_ID	- pointer to device control structure
 * Description:
 * NOTE:
 *  Care should be taken to avoid calling this function if other tasks
 *  have been given this device ID. Generally, this function should never
 *  need to be called. Mv300DevCreate would be called during system
 *  initialization, and all tasks which use the interface would be passed
 *  the device_id. This function would only be called if a graceful shutdown
 *  is required.
 */
STATUS Mv300DevForget (devId)
     MV300_DEV_ID devId;
{
  BYTE              commands[4];
  UINT		    bufLen;
  UINT		    bytesWritten;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* clear device */
  if (devId->config.secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | devId->config.secAddr;
      commands[2] = MV300_BRD_K_GPIB_SDC;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SDC;
      commands[2] = MV300_BRD_K_GPIB_UNL;
      bufLen = 3;
    }
  Mv300BrdDevCommand (devId->brdId, &(devId->config),
		      commands, bufLen, &bytesWritten);

  /* free structure and return */
  semGive (devId->devSemId);
  semDelete (devId->devSemId);
  return (free ((ADDR) devId));
}/* Mv300DevForget*/

/*========================= Device Functions =================================*/
/*-----------------------------------------------------------------------------
 * Mv300DevBrdIdGet (devId, pBrdId) : status
 *
 * Purpose:
 *  Get board Id.
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  pBrdId		: MV300_BRD_ID*  - pointer to destination variable
 * Description:
 *  The board Id refers to the MVME300 board control structure which
 *  drives the IEEE488 bus that is connected to this device. This Id is
 *  required for direct communication with the Mv300Brd module functions.
 */
STATUS Mv300DevBrdIdGet (devId, pBrdId)
     MV300_DEV_ID   devId;
     MV300_BRD_ID  *pBrdId;
{
  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);
  
  /* return board Id */
  *pBrdId = devId->brdId;

  /* release control and return */
  semGive (devId->devSemId);
  return (OK);
} /*Mv300DevBrdIdGet*/

/*-----------------------------------------------------------------------------
 * Mv300DevBrdIdSet (devId, brdId) : status
 *
 * Purpose:
 *  Set board Id.
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  brdId		: MV300_BRD_ID	 - board control structure ID
 * Description:
 *  The board Id refers to the MVME300 board control structure which
 *  drives the IEEE488 bus that is connected to this device. This Id is
 *  required for direct communication with the Mv300Brd module functions.
 * Note:
 *  Changing the board Id should be done only if the device has been
 *  moved to a new IEEE488 bus. Generally the user should not change this.
 */
STATUS Mv300DevBrdIdSet (devId, brdId)
     MV300_DEV_ID   devId;
     MV300_BRD_ID   brdId;
{
  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* update board Id */
  devId->brdId = brdId;

  /* restore control of device and return */
  semGive (devId->devSemId);
  return (OK);
} /*Mv300DevBrdIdSet*/

/*-----------------------------------------------------------------------------
 * Mv300DevClear (devId) : status
 *
 * Purpose:
 *  Send selective device clear command to device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 * Description:
 */
STATUS Mv300DevClear (devId)
     MV300_DEV_ID   devId;
{
  STATUS  status = OK;
  BYTE    commands[4];
  UINT	  bufLen;
  UINT	  bytesWritten;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* clear device */
  if (devId->config.secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | devId->config.secAddr;
      commands[2] = MV300_BRD_K_GPIB_SDC;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SDC;
      commands[2] = MV300_BRD_K_GPIB_UNL;
      bufLen = 3;
    }
  status = Mv300BrdDevCommand (devId->brdId, &(devId->config),
			       commands, bufLen, &bytesWritten);

  /* release control and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevClear*/

/*-----------------------------------------------------------------------------
 * Mv300DevConfRd (devId, pConf) : status
 *
 * Purpose:
 *  Read device configuration
 * Returns:
 *  status	   : STATUS	   - OK if no problems
 * Arguments:			
 *  devId	   : MV300_DEV_ID  - device control structure ID
 *  pConf	   : PMV300_CONFIG - pointer to device configuration structure.
 * Description:
 *  Read the current device configuration parameters
 */
STATUS Mv300DevConfRd (devId, pConf)
     MV300_DEV_ID   devId;
     PMV300_CONFIG  pConf;
{
  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* copy board configuration */
  pConf->timeout	= devId->config.timeout;
  pConf->flags		= devId->config.flags;
  pConf->priAddr	= devId->config.priAddr;
  pConf->secAddr	= devId->config.secAddr;
  pConf->reosChar	= devId->config.reosChar;
  pConf->weosChar	= devId->config.weosChar;

  /* release control and return */
  semGive (devId->devSemId);
  return (OK);
} /*Mv300DevConfRd*/

/*-----------------------------------------------------------------------------
 * Mv300DevConfWrt (devId, pConf) : status
 *
 * Purpose:
 *  Write device configuration
 * Returns:
 *  status	   : STATUS	   - OK if no problems
 * Arguments:			
 *  devId	   : MV300_DEV_ID  - device control structure ID
 *  pConf	   : PMV300_CONFIG - pointer to device configuration structure.
 * Description:
 *  Update the device configuration parameter.
 */
STATUS Mv300DevConfWrt (devId, pConf)
     MV300_DEV_ID   devId;
     PMV300_CONFIG  pConf;
{
  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* copy board configuration */
  devId->config.timeout		= pConf->timeout;
  devId->config.flags		= pConf->flags;
  devId->config.priAddr		= pConf->priAddr;
  devId->config.secAddr		= pConf->secAddr;
  devId->config.reosChar	= pConf->reosChar;
  devId->config.weosChar	= pConf->weosChar;

  /* release control and return */
  semGive (devId->devSemId);
  return (OK);
} /*Mv300DevConfWrt*/

/*-----------------------------------------------------------------------------
 * Mv300DevLocal (devId) : status
 *
 * Purpose:
 *  Enable local control for a device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 * Description:
 *  Place the device in local program mode by sending the Go To Local (GTL)
 *  command. REN is asserted at startup, so devices are automatically placed
 *  in remote program mode. This function is required to place the device
 *  back in local mode.
 */
STATUS Mv300DevLocal (devId)
     MV300_DEV_ID   devId;
{
  STATUS  status = OK;
  BYTE    commands[4];
  UINT	  bufLen;
  UINT	  bytesWritten;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* place device in local mode */
  if (devId->config.secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | devId->config.secAddr;
      commands[2] = MV300_BRD_K_GPIB_GTL;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_GTL;
      commands[2] = MV300_BRD_K_GPIB_UNL;
      bufLen = 3;
    }
  status = Mv300BrdDevCommand (devId->brdId, &(devId->config),
			       commands, bufLen, &bytesWritten);

  /* release control and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevLocal*/

/*-----------------------------------------------------------------------------
 * Mv300DevPPoll (devId, pResult) : status
 *
 * Purpose:
 *  Parallel poll a device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  pResult		: BYTE*		 - pointer to result
 * Description:
 *  Conduct a parallel poll and return the parallel poll response.
 */
STATUS Mv300DevPPoll (devId, pResult)
     MV300_DEV_ID   devId;
     BYTE	   *pResult;
{
  STATUS	status = OK;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* parallel poll the bus */
  status = Mv300BrdPPoll (devId->brdId, pResult);

  /* free structure and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevPPoll*/

/*-----------------------------------------------------------------------------
 * Mv300DevPPollByte (devId, confByte, state) : status
 *
 * Purpose:
 *  Configure/Unconfigure a device for a parallel poll
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  confByte		: BYTE		 - parallel poll configuration byte
 *  state		: BOOL		 - TRUE = configure; FALSE = unconfigure
 * Description:
 *  Configure the device for parallel polls. The configuration byte specifies
 *  the IEEE488 data line and sense that the device must use when responding
 *  to the identify (IDY) message during a parallel poll. Valid configuration
 *  bytes are in the range 0x00-0x0F where bit 7 (0x08) is the ist bit. The
 *  higher bits are masked off before sending. If state is FALSE, the device
 *  is unconfigured for parallel poll.
 */
STATUS Mv300DevPPollByte (devId, confByte, state)
     MV300_DEV_ID   devId;
     BYTE	    confByte;
     BOOL	    state;
{
  STATUS  status = OK;
  BYTE    data;
  BYTE    commands[5];
  UINT	  bufLen;
  UINT	  bytesWritten;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* test for configure */
  if (state != FALSE)
    {
      data = MV300_BRD_K_GPIB_PPE | (0x0f & confByte);
    }
  else
    {
      data = MV300_BRD_K_GPIB_PPD;
    }
  /* send commands */
  if (devId->config.secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | devId->config.secAddr;
      commands[2] = MV300_BRD_K_GPIB_PPC;
      commands[3] = data;
      commands[4] = MV300_BRD_K_GPIB_UNL;
      bufLen = 5;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_PPC;
      commands[2] = data;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  status = Mv300BrdDevCommand (devId->brdId, &(devId->config),
			       commands, bufLen, &bytesWritten);

  /* release control and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevPPollConf*/

/*-----------------------------------------------------------------------------
 * Mv300DevRead (devId, pBuf, bufLen, pBytesRead) : status
 *
 * Purpose:
 *  Read device data from GPIB device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - pointer to device control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesRead		: UINT*		 - pointer to variable for number
 * Description:
 *  Address device to talk, board to listen, and transfer data into buffer.
 *  Return number of bytes read in variable pointed to by pBytesRead.
 */
STATUS Mv300DevRead (devId, pBuf, bufLen, pBytesRead)
     MV300_DEV_ID         devId;
     BYTE*		  pBuf;
     UINT		  bufLen;
     UINT*		  pBytesRead;
{
  STATUS	status = OK;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* call board function to perform device read */
  status = Mv300BrdDevRead (devId->brdId, &(devId->config),
			    pBuf, bufLen, pBytesRead);

  /* free structure and return */
  semGive (devId->devSemId);
  return (status);
}/*Mv300DevRead*/

/*-----------------------------------------------------------------------------
 * Mv300DevSPoll (devId, pResult) : status
 *
 * Purpose:
 *  Serial poll a device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  pResult		: BYTE*		 - pointer to result
 * Description:
 *  Serial poll device and return result in pResult.
 */
STATUS Mv300DevSPoll (devId, pResult)
     MV300_DEV_ID   devId;
     BYTE	   *pResult;
{
  STATUS	status = OK;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* call board function to perform poll */
  status = Mv300BrdDevSPoll (devId->brdId, &devId->config, pResult);

  /* free structure and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevSPoll*/

/*-----------------------------------------------------------------------------
 * Mv300DevStatus (devId, pStatus) : status
 *
 * Purpose:
 *  Get current MVME300 board status
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 *  pStatus		: PMV300_STATUS  - pointer to board status
 * Description:
 *  load MV300_STATUS variable with board status register and state info.
 * Note:
 *  This is NOT a device status. This is device specific, hence need
 *  individual device read/write sequence to find device information.
 */
STATUS Mv300DevStatus (devId, pStatus)
     MV300_DEV_ID   devId;
     PMV300_STATUS  pStatus;
{
  STATUS	status = OK;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* call Mv300Brd module to get status */
  status = Mv300BrdStatus (devId->brdId, pStatus);

  /* free structure and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevStatus*/

/*-----------------------------------------------------------------------------
 * Mv300DevTrigger (devId) : status
 *
 * Purpose:
 *  Trigger device
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID	 - device control structure ID
 * Description:
 *  Send Group Execute Trigger (GET) command to the indicated device.
 */
STATUS Mv300DevTrigger (devId)
     MV300_DEV_ID   devId;
{
  STATUS  status = OK;
  BYTE    commands[4];
  UINT	  bufLen;
  UINT	  bytesWritten;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* send GET to device */
  if (devId->config.secAddr != 0)
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_SCG | devId->config.secAddr;
      commands[2] = MV300_BRD_K_GPIB_GET;
      commands[3] = MV300_BRD_K_GPIB_UNL;
      bufLen = 4;
    }
  else
    {
      commands[0] = MV300_BRD_K_GPIB_LAG | devId->config.priAddr;
      commands[1] = MV300_BRD_K_GPIB_GET;
      commands[2] = MV300_BRD_K_GPIB_UNL;
      bufLen = 3;
    }
  status = Mv300BrdDevCommand (devId->brdId, &(devId->config),
			       commands, bufLen, &bytesWritten);

  /* release control and return */
  semGive (devId->devSemId);
  return (status);
} /*Mv300DevTrigger*/

/*-----------------------------------------------------------------------------
 * Mv300DevWrite (devId, pBuf, bufLen, pBytesWritten) : status
 *
 * Purpose:
 *  Write to device
 * Returns:
 *  status		: STATUS	- OK if no problems
 * Arguments:			
 *  devId		: MV300_DEV_ID 	 - pointer to device control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesWritten	: UINT*		 - pointer to variable for
 * Description:
 *  Address device to listen, board to talk, and transfer data from buffer.
 *  Return number of bytes written in variable *pBytesWritten.
 */
STATUS Mv300DevWrite (devId, pBuf, bufLen, pBytesWritten)
     MV300_DEV_ID       devId;
     BYTE*		pBuf;
     UINT		bufLen;
     UINT*		pBytesWritten;
{
  STATUS	status = OK;

  /* test for valid device id */
  if (devId == NULL)
    return (ERROR_EDID);

  /* else grab control */
  semTake (devId->devSemId);

  /* call board function to perform device read */
  status = Mv300BrdDevWrite (devId->brdId, &(devId->config),
			     pBuf, bufLen, pBytesWritten);

  /* free structure and return */
  semGive (devId->devSemId);
  return (status);
}/*Mv300DevWrite*/

/*========================== Local Functions =================================*/
/*=============================== End ========================================*/

