/* $Id: Mv300Drv.c,v 1.2 91/02/06 11:40:41 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300Drv.c $
 *
 * $Revision: 1.2 $
 *
 * $State: Exp $
 *
 * $Author: cwcork $
 *
 * $Log:	Mv300Drv.c,v $
 * Revision 1.2  91/02/06  11:40:41  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.1  90/08/08  16:21:01  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *   This is the driver for the Motoroal MVME300 IEEE-488 bus controller module.
 *
 * FUNCTIONS:
 *  Exported-
 *    --Create and Forget--
 *  Local-
 *
 * COMMENTS:
 *  This module actually contains two device drivers, one for the MVME300 module
 *  and one for the IEEE-488 devices which are controlled by the module. Both
 *  drivers are initialized together, however devices are added separately.
 *  Because the device structures are dynamically allocated, we have added
 *  the exported Forget routines (Mv300DrvBrdForget and Mv300DrvDevForget)
 *  to free up unused memory on the heap.
 *  
 *  USER CALLABLE ROUTINES
 *    Most of the routines in this driver are accessible only through the I/O
 *    system.  Five routines, however, must be called directly: Mv300Drv () to
 *    initialize the drivers, Mv300DrvBrdCreate () to create the board devices,
 *    Mv300DrvBrdForget () to destroy the board devices, Mv300DrvDevCreate () to
 *    create the IEEE488 devices, and Mv300DrvDevForget () to destroy the
 *    IEEE488 devices.
 *    
 *    Before using the driver, it must be initialized by calling the routine:
 *    .CS
 *     STATUS Mv300Drv ()
 *    .CE
 *    The first time Mv300Drv is called, it will install the Mv300DrvBrd and
 *    Mv300DrvDev drivers. Subsequent calls have no effect.
 *    This routine should be called exactly once, before any reads, writes, or
 *    Mv300DevCreates.  Normally, it is called from usrRoot (2) in usrConfig.c.
 *    
 *    CREATING MV300_BRD DEVICES
 *    Before any Mv300 board can be used, it must be created. This is done
 *    with the Mv300DrvBrdCreate call. Each MV300 board should have exactly
 *    one device associated with it, by calling this routine.
 *    .CS
 *     STATUS Mv300DrvBrdCreate (name, ioBase, intNum, intLevel)
 *         CHAR *name;     Name to use for this device
 *	   ADDR  ioBase;   Base address of board
 *	   INT	 intNum;   Interrupt vector number.
 *	   INT	 intLevel; Interrupt level.
 *    .CE
 *    
 *    For instance, to create the device "/gpib0",
 *    with base address 0xffff0100, interrupt vector number 300,
 *    and interrupt level 3, the proper call would be:
 *    .CS
 *       Mv300DrvBrdCreate ("/gpib", 0xffff0100, 300, 3);
 *    .CE
 *
 *    CREATING MV300_DEV DEVICES
 *    Before any Mv300 IEEE488 device can be used, it must be created.
 *    This is done with the Mv300DrvDevCreate call.
 *    Each MV300 IEEE488 device should have exactly one device structure
 *    associated with it, by calling this routine.
 *    .CS
 *     STATUS Mv300DrvDevCreate (name, priAddr, secAddr)
 *         CHAR *name;     Name to use for this device
 *	   BYTE  priAddr   Primary IEEE488 address.
 *	   BYTE  secAddr   Secondary IEEE488 address.
 *    .CE
 *    
 *    For instance, to create the device "/gpib0/22",
 *    with primary address 22 and secondary address 0, the proper call would be:
 *    .CS
 *       Mv300DrvDevCreate ("/gpib/22", 22, 0);
 *    .CE
 *
 *    IOCTL
 *    **** Description of ioctl codes for Mv300 devices ****
 *
 * KNOWN BUGS:
 */
/*========================== External Declarations ===========================*/
#include "vxWorks.h"
#include "ioLib.h"
#include "iosLib.h"
#include "Mv300.h"
#include "Mv300Dev.h"
#include "Mv300Brd.h"
/*========================== Local Declarations ==============================*/
/* Board Device structure */
typedef struct
{
  DEV_HDR	devHdr;
  MV300_BRD_ID  brdId;
} MV300_BRD_DEV;

typedef MV300_BRD_DEV	*PMV300_BRD_DEV;

/* IEEE488 Device structure */
typedef struct
{
  DEV_HDR	devHdr;
  MV300_DEV_ID  devId;
} MV300_488_DEV;

typedef MV300_488_DEV	*PMV300_488_DEV;

/*========================== Local Macros ====================================*/
/*========================== Local Functions Prototypes ======================*/
LOCAL INT     mv300DrvBrdOpen ();
LOCAL STATUS  mv300DrvBrdClose ();
LOCAL INT     mv300DrvBrdRead ();
LOCAL INT     mv300DrvBrdWrite ();
LOCAL INT     mv300DrvBrdIoctl ();
LOCAL INT     mv300DrvDevOpen ();
LOCAL STATUS  mv300DrvDevClose ();
LOCAL INT     mv300DrvDevRead ();
LOCAL INT     mv300DrvDevWrite ();
LOCAL INT     mv300DrvDevIoctl ();
/*========================== Local Variables =================================*/
LOCAL INT	mv300DrvBrdNum = 0;
LOCAL INT	mv300DrvDevNum = 0;
LOCAL BOOL	mv300DrvCreated = FALSE;
/*========================== Exported Functions ==============================*/
/*-----------------------------------------------------------------------------
 * Mv300Drv () : STATUS
 *
 * Purpose:
 *  Install MVME300 board and IEEE488 device drivers.
 * Inputs:
 * Outputs:
 * Returns:
 *  status	: OK, or ERROR if non-existent hardware or ran out of memory.
 * Description:
 *  The first time called will install drivers.
 */

STATUS Mv300Drv ()
{
  /* Create driver first time called */
  if ( !mv300DrvCreated )
    {
      mv300DrvBrdNum = iosDrvInstall (mv300DrvBrdOpen, mv300DrvBrdClose,
				      mv300DrvBrdOpen, mv300DrvBrdClose,
				      mv300DrvBrdRead, mv300DrvBrdWrite,
				      mv300DrvBrdIoctl);
      mv300DrvDevNum = iosDrvInstall (mv300DrvDevOpen, mv300DrvDevClose,
				      mv300DrvDevOpen, mv300DrvDevClose,
				      mv300DrvDevRead, mv300DrvDevWrite,
				      mv300DrvDevIoctl);
      mv300DrvCreated = TRUE;
    }
  /* return ERROR if either driver number is bad */
  if ((mv300DrvBrdNum == ERROR) || (mv300DrvDevNum == ERROR))
    return (ERROR);
  else
    return (OK);
}/*Mv300Drv*/

/*------------------------------------------------------------------------------
 *
 * Mv300DrvBrdCreate (name, ioBase, intNum, intLevel) : status
 *
 * Purpose:
 *  Create board device structure and install device.
 * Inputs:
 *  name	: char*		- name of board device.
 *  ioBase	: char*		- base i/o address.
 *  intNum	: int		- interrupt vector number.
 *  intLevel	: int		- interrupt level.
 * Outputs:
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Description:
 */
STATUS Mv300DrvBrdCreate (name, ioBase, intNum, intLevel)
     char      *name;
     char      *ioBase;
     int	intNum;
     int	intLevel;
{
  STATUS	  status = OK;
  PMV300_BRD_DEV  pBrdDev;
  MV300_CONFIG	  brdConfig;

  /* try to create device control structure */
  if ((pBrdDev = (PMV300_BRD_DEV) malloc (sizeof (MV300_BRD_DEV))) == NULL)
    {
      return (MV300$_EMEM);
    }

  /* initialize default configuration structure */
  brdConfig.timeout  = 600;		/* default timeout is 10 seconds */
  brdConfig.flags    = 0;		/* no REOS, WEOS, DMA, or EOT */
  brdConfig.priAddr  = 0;		/* not used by Mv300BrdCreate */
  brdConfig.secAddr  = 0;		/* not used by Mv300BrdCreate */
  brdConfig.reosChar = '\0';		/* default null terminator */
  brdConfig.weosChar = '\0';		/* default null terminator */

  /* create board Id structure and initialize board hardware*/
  status = Mv300BrdCreate (&pBrdDev->brdId,
			   ioBase, intNum, intLevel, &brdConfig);

  /* if status OK, then add device to system list */
  if (status == OK)
    status = iosDevAdd ((DEV_HDR *) pBrdDev, name, mv300DrvBrdNum);

  return (status);
}/*Mv300DrvBrdCreate*/

/*------------------------------------------------------------------------------
 *
 * Mv300DrvBrdForget (name) : status
 *
 * Purpose:
 *  Delete board device from i/o system.
 * Inputs:
 *  name	: char*		- name of board device.
 * Outputs:
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Description:
 */
STATUS Mv300DrvBrdForget (name)
     char	*name;
{
  STATUS	 status = OK;
  PMV300_BRD_DEV pBrdDev = NULL;
  char	        *pNameTail = NULL;

  /* find device header */
  if ((pBrdDev = (PMV300_BRD_DEV) iosDevFind (name, &pNameTail)) == NULL)
    return (ERROR);
  /* free board control structure and release board */
  if ((status = Mv300BrdForget (pBrdDev->brdId)) != OK)
    return (status);
  /* remove device */
  iosDevDelete (pBrdDev);
  /* free device structure and return */
  return (free ((char *)pBrdDev));
}/*Mv300DrvBrdForget*/

/*------------------------------------------------------------------------------
 *
 * Mv300DrvDevCreate (name, priAddr, secAddr) : status
 *
 * Purpose:
 *  Create IEEE488 device structure and install device
 * Inputs:
 *  name	: char*		- name of IEEE488 device.
 *  priAddr	: char		- primary address of IEEE488 device.
 *  secAddr	: char		- secondary address of IEEE488 device.
 * Outputs:
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Description:
 *  Create IEEE488 device structure using default configuration settings
 *  plus priAddr and secAddr information. The associated board device must
 *  already have been installed and its name must be a substring of 'name'.
 *  e.g. If the 488 device name is "/gpib0/dvm", then the board device name
 *  must be "/gpib0/".
 * Note:
 *  This routine uses iosDevFind to find the associated board device structure.
 *  iosDevFind finds the first device whose name begins with the board device
 *  name. This should work because the board device must be created before any
 *  488 devices can be created; however, I am a bit uncomfortable about
 *  relying on this match technique.
 */
STATUS Mv300DrvDevCreate (name, priAddr, secAddr)
     char      *name;
     char	priAddr;
     char	secAddr;
{
  STATUS	  status = OK;
  char		  brdName[80];
  char		  devName[80];
  char		 *pNameTail;
  PMV300_BRD_DEV  pBrdDev;
  PMV300_488_DEV  p488Dev;
  MV300_CONFIG	  devConfig;

  /* First try to find board device */
  pathSplit (name, brdName, devName);
  pBrdDev = (PMV300_BRD_DEV) iosDevFind (brdName, &pNameTail);
  if ((pBrdDev == NULL) || ((strlen (brdName) +1) != (pNameTail - name)))
    {
      return (ERROR);
    }
  
  /* try to create device control structure */
  if ((p488Dev = (PMV300_488_DEV) malloc (sizeof (MV300_488_DEV))) == NULL)
    {
      return (MV300$_EMEM);
    }

  /* initialize default configuration structure */
  devConfig.timeout  = 600;		/* default timeout is 10 seconds */
  devConfig.flags    = 0;		/* no REOS, WEOS, DMA, or EOT */
  devConfig.priAddr  = priAddr;		/* primary 488 address */
  devConfig.secAddr  = secAddr;		/* secondary 488 address */
  devConfig.reosChar = '\0';		/* default null terminator */
  devConfig.weosChar = '\0';		/* default null terminator */

  /* create board Id structure and initialize board hardware*/
  status = Mv300DevCreate (&p488Dev->devId, pBrdDev->brdId, &devConfig);

  /* if status OK, then add device to system list */
  if (status == OK)
    status = iosDevAdd ((DEV_HDR *) p488Dev, name, mv300DrvBrdNum);

  return (status);
}/*Mv300DrvDevCreate*/

/*------------------------------------------------------------------------------
 *
 * Mv300DrvDevForget (name) : status
 *
 * Purpose:
 *  Delete IEEE488 device from i/o system.
 * Inputs:
 *  name	: char*		- name of IEEE488 device
 * Outputs:
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Description:
 */
STATUS Mv300DrvDevForget (name)
     char      *name;
{
  STATUS	 status = OK;
  PMV300_488_DEV p488Dev = NULL;
  char	        *pNameTail = NULL;

  /* find device header */
  if ((p488Dev = (PMV300_BRD_DEV) iosDevFind (name, &pNameTail)) == NULL)
    return (ERROR);
  /* free board control structure and release board */
  if ((status = Mv300BrdForget (p488Dev->devId)) != OK)
    return (status);
  /* remove device */
  iosDevDelete (p488Dev);
  /* free device structure and return */
  return (free ((char *)p488Dev));
}/*Mv300DrvDevForget*/

/*========================== Local Functions =================================*/

/*-------------------------- Board Device Functions --------------------------*/
/*------------------------------------------------------------------------------
 * mv300DrvBrdOpen (pBrdDev, name, mode) : id
 *
 * Purpose:
 *  Open MVME300 board device interface
 * Inputs:
 *  pBrdDev	: PMV300_DRV_BRD	- pointer to board device structure.
 *  name	: char*			- name of board device.
 *  mode	: int			- mode of device (ignored).
 * Outputs:
 * Returns:
 *  id		: int			- board device id = (int) pBrdDev.
 * Description:
 */
LOCAL int mv300DrvBrdOpen (pBrdDev, name, mode)
     PMV300_DEV_BRD pBrdDev;
     char	   *name;
     int	    mode;
{
  /* just make sure that the board is online */
  Mv300BrdOnline (pBrdDev->brdId);
  /* return a pointer to device structure */
  return ((int) pBrdDev);
}/*mv300DrvBrdOpen*/

/*------------------------------------------------------------------------------
 * mv300DrvBrdClose (pBrdDev) : status
 *
 * Purpose:
 *  Close MVME300 board device
 * Inputs:
 *  pBrdDev	: PMV300_DRV_BRD	- pointer to board device structure.
 * Outputs:
 * Returns:
 *  status	: STATUS		- OK, if no problems
 * Description:
 */
LOCAL STATUS mv300DrvBrdClose (pBrdDev)
     PMV300_DRV_BRD pBrdDev;
{
  /* just make the board offline */
  return (Mv300BrdOffline (pBrdDev->brdId));
}/*mv300DrvBrdClose*/

/*------------------------------------------------------------------------------
 * mv300DrvBrdRead (pBrdDev, buffer, maxbytes) : bytesRead
 *
 * Purpose:
 *  Read from board device
 * Inputs:
 *  pBrdDev	: PMV300_DRV_BRD	- pointer to board device structure.
 *  buffer	: char*			- pointer to read buffer.
 *  maxbytes    : int			- maximum number of bytes for transfer.
 * Outputs:
 * Returns:
 *  bytesRead	: int			- number of bytes read
 * Description:
 */
LOCAL int mv300DrvBrdRead (pBrdDev, buffer, maxbytes)
     PMV300_DRV_BRD pBrdDev;
     char	   *buffer;
     int	    maxbytes;
{
  UINT		bytesRead;

  if (Mv300BrdRead (pBrdDev->brdId, buffer, maxbytes, &bytesRead) != OK)
    return (ERROR);
  else
    return (bytesRead);
}/*mv300DrvBrdRead*/

/*------------------------------------------------------------------------------
 * mv300DrvBrdWrite (pBrdDev, buffer, nbytes) : bytesWritten
 *
 * Purpose:
 *  Write to board device
 * Inputs:
 *  pBrdDev	: PMV300_DRV_BRD	- pointer to board device structure.
 *  buffer	: char*			- pointer to write buffer.
 *  nbytes      : int			- number of bytes for transfer.
 * Outputs:
 * Returns:
 *  bytesWritten: int			- number of bytes written.
 * Description:
 */
LOCAL int mv300DrvBrdWrite (pBrdDev, buffer, nbytes)
     PMV300_DRV_BRD pBrdDev;
     char	   *buffer;
     int	    nbytes;
{
  UINT	bytesWritten;

  /* call board write routine */
  if (Mv300BrdWrite (pBrdDev->brdId, buffer, nbytes, &bytesWritten) != OK)
    return (ERROR);
  else
    return (bytesWritten);
}/*mv300DrvBrdWrite*/

/*------------------------------------------------------------------------------
 * mv300DrvBrdIoctl (pBrdDev, request, arg) : result
 *
 * Purpose:
 *  Board device control function
 * Inputs:
 *  pBrdDev	: PMV300_DRV_BRD	- pointer to board device structure.
 *  request	: int			- control function request.
 *  arg		: int			- arbitrary argument.
 * Outputs:
 * Returns:
 *  result	: int			- status result.
 * Description:
 */
LOCAL int mv300DrvBrdIoctl (pBrdDev, request, arg)
     PMV300_DRV_BRD pBrdDev;
     int	    request;
     int	    arg;
{
  FAST MV300_BRD_ID brdId  = pBrdDev->brdId;
  STATUS 	    status = OK;

  
}/*mv300DrvBrdIoctl*/

/*-------------------------- IEEE488 Device Functions --------------------------*/
/*------------------------------------------------------------------------------
 * mv300DrvDevOpen (p488Dev, name, mode) : id
 *
 * Purpose:
 *  Open MVME300 IEEE488 Device interface
 * Inputs:
 *  p488Dev	: PMV300_DRV_BRD	- pointer to IEEE488 Device structure.
 *  name	: char*			- name of IEEE488 Device.
 *  mode	: int			- mode of device (ignored).
 * Outputs:
 * Returns:
 *  id		: int			- IEEE488 Device id = (int) p488Dev.
 * Description:
 */
LOCAL int mv300DrvDevOpen (p488Dev, name, mode)
     PMV300_DEV_BRD p488Dev;
     char	   *name;
     int	    mode;
{

  return ((int) p488Dev);
}/*mv300DrvDevOpen*/

/*------------------------------------------------------------------------------
 * mv300DrvDevClose (p488Dev) : status
 *
 * Purpose:
 *  Close MVME300 IEEE488 Device
 * Inputs:
 *  p488Dev	: PMV300_DRV_BRD	- pointer to IEEE488 Device structure.
 * Outputs:
 * Returns:
 *  status	: STATUS		- OK, if no problems
 * Description:
 */
LOCAL STATUS mv300DrvDevClose (p488Dev)
     PMV300_DRV_BRD p488Dev;
{

}/*mv300DrvDevClose*/

/*------------------------------------------------------------------------------
 * mv300DrvDevRead (p488Dev, buffer, maxbytes) : bytesRead
 *
 * Purpose:
 *  Read from IEEE488 Device
 * Inputs:
 *  p488Dev	: PMV300_DRV_BRD	- pointer to IEEE488 Device structure.
 *  buffer	: char*			- pointer to read buffer.
 *  maxbytes    : int			- maximum number of bytes for transfer.
 * Outputs:
 * Returns:
 *  bytesRead	: int			- number of bytes read
 * Description:
 */
LOCAL int mv300DrvDevRead (p488Dev, buffer, maxbytes)
     PMV300_DRV_BRD p488Dev;
     char	   *buffer;
     int	    maxbytes;
{
}/*mv300DrvDevRead*/

/*------------------------------------------------------------------------------
 * mv300DrvDevWrite (p488Dev, buffer, nbytes) : bytesWritten
 *
 * Purpose:
 *  Write to IEEE488 Device
 * Inputs:
 *  p488Dev	: PMV300_DRV_BRD	- pointer to IEEE488 Device structure.
 *  buffer	: char*			- pointer to write buffer.
 *  nbytes      : int			- number of bytes for transfer.
 * Outputs:
 * Returns:
 *  bytesWritten: int			- number of bytes written.
 * Description:
 */
LOCAL int mv300DrvDevWrite (p488Dev, buffer, nbytes)
     PMV300_DRV_BRD p488Dev;
     char	   *buffer;
     int	    nbytes;
{
}/*mv300DrvDevWrite*/

/*------------------------------------------------------------------------------
 * mv300DrvDevIoctl (p488Dev, request, arg) : result
 *
 * Purpose:
 *  IEEE488 Device control function
 * Inputs:
 *  p488Dev	: PMV300_DRV_BRD	- pointer to IEEE488 Device structure.
 *  request	: int			- control function request.
 *  arg		: int			- arbitrary argument.
 * Outputs:
 * Returns:
 *  result	: int			- status result.
 * Description:
 */
LOCAL int mv300DrvDevIoctl (p488Dev, request, arg)
     PMV300_DRV_BRD p488Dev;
     int	    request;
     int	    arg;
{
}/*mv300DrvDevIoctl*/

/*============================== End =========================================*/

